@@ -56,7 +56,7 @@ GAsyncQueue *js_queue;
 static gboolean queue_available = FALSE;
 
 static JSBool
-show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
+show_about (JSContext *cx, uintN argc, jsval *vp)
 {
   static const gchar *const authors[] = {
     "Darren Salt <dsalt@users.sourceforge.net>",
@@ -126,7 +126,7 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
   }
 
   window_show (about, NULL);
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -184,8 +184,10 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
   if ((str = se_result_str (gse)))
   {
     /* JS output. Could end up in a terminal or the JS console. */
+    char *free_str = str;
     str = g_strdup_printf (_("result: %s"), str);
     cb (cb_data, "%s", str);
+    free (free_str);
     free (str);
   }
   else if (se_result_double (gse, &num.d))
@@ -405,12 +407,12 @@ se_startup_response_cb (GtkWidget *widget, int response, gpointer data)
 }
 
 static JSBool
-show_startup (JSContext *cx, JSObject *obj, uintN argc,
-	      jsval *argv, jsval *rval)
+show_startup (JSContext *cx, uintN argc, jsval *vp)
 {
   if (!GTK_WIDGET_VISIBLE (se_startup_window))
     reset_startup_buffer (se_startup_cmds);
   window_show (se_startup_window, NULL);
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -439,8 +441,8 @@ void engine_init (void)
   gse = se_new ();
 
   static const se_f_def_t defs[] = {
-    { "about_show", show_about, 0, 0, SE_GROUP_DIALOGUE, NULL, NULL },
-    { "startup_cmds_show", show_startup, 0, 0, SE_GROUP_DIALOGUE, NULL, NULL },
+    { "about_show", show_about, 0, JSFUN_FAST_NATIVE, SE_GROUP_DIALOGUE, NULL, NULL },
+    { "startup_cmds_show", show_startup, 0, JSFUN_FAST_NATIVE, SE_GROUP_DIALOGUE, NULL, NULL },
     { NULL }
   };
   se_defuns (gse, gse->g, defs);
