@@ -34,12 +34,12 @@
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
-#include <pwd.h>
 #include <sys/types.h>
 #include <pthread.h>
 #include <sched.h>
 #include <sys/time.h>
 
+#ifdef HAVE_X11
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <X11/cursorfont.h>
@@ -47,6 +47,7 @@
 #ifdef HAVE_DPMS_EXTENSION
 #include <X11/extensions/dpms.h>
 #endif
+#endif
 #ifdef HAVE_XTESTEXTENSION
 #include <X11/extensions/XTest.h>
 #endif
@@ -75,21 +76,27 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkmain.h>
+#if defined(GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
+#elif defined(GDK_WINDOWING_WIN32)
+#include <windows.h>
+#include <gdk/gdkwin32.h>
+#endif
 #include <xine.h>
 /* for get_capabilities() */
 #include <xine/video_out.h>
 
 #include "gtkvideo.h"
-#include "globals.h"
 #include "post.h"
 #include "playlist.h"
 #include "utils.h"
 
+#ifdef HAVE_X11
 /* missing stuff from X includes */
 #ifndef XShmGetEventBase
 extern int XShmGetEventBase(Display *);
 #endif
+#endif
 
 static void gtk_video_class_init    (GtkVideoClass   *klass);
 static void gtk_video_instance_init (GtkVideo        *gvideo);
@@ -109,7 +116,9 @@ static void gtk_video_size_allocate (GtkWidget      *widget,
 static gboolean gtv_unblank_screen (GtkVideo *);
 static gboolean gtv_update_resize_factor (GtkVideo *);
 
+#ifdef SCREENSAVER_COMMANDS
 static int screensaver_poke (const char *const [], const char *);
+#endif
 #ifdef XDG_SCREENSAVER_COMMAND
 static int xdg_screensaver_poke (const char *, Window);
 #endif
@@ -147,8 +156,13 @@ struct gtk_video_private_s {
   char                    *video_driver_id;
   xine_video_port_t       *video_port;
 
+#if defined(GDK_WINDOWING_X11)
   Window                   video_window;
-  pthread_t                thread;
+#elif defined(GDK_WINDOWING_WIN32)
+  HWND                     video_window;
+  win32_visual_t           win32_visual; /* XXX: better to code frame callbacks
+                                                 to vo plugin */
+#endif
 
   double                   display_ratio;
 
@@ -174,7 +188,7 @@ struct gtk_video_private_s {
 
   gboolean		   fullscreen_mode:8, pointer_visible:8;
   GtkRequisition	   fullscreen;
-  Cursor		   pointer[3];
+  GdkCursor		   *pointer[3];
   uint8_t		   pointer_shape, pointer_shape_old;
   struct {
     struct timeval	   time;
@@ -238,16 +252,17 @@ static inline GdkDisplay *gtv_get_display (GtkVideo *gtv)
   return gdk_drawable_get_display ((GdkDrawable *)gtv->widget.window);
 }
 
-static inline Display *gtv_get_xdisplay (GtkVideo *gtv)
-{
-  return GDK_WINDOW_XDISPLAY (gtv->widget.window);
-}
-
 static inline GdkScreen *gtv_get_screen (GtkVideo *gtv)
 {
   return gdk_drawable_get_screen ((GdkDrawable *)gtv->widget.window);
 }
 
+#ifdef HAVE_X11
+static inline Display *gtv_get_xdisplay (GtkVideo *gtv)
+{
+  return GDK_WINDOW_XDISPLAY (gtv->widget.window);
+}
+
 static inline Screen *gtv_get_xscreen (GtkVideo *gtv)
 {
   return GDK_SCREEN_XSCREEN (gtv_get_screen (gtv));
@@ -257,6 +272,7 @@ static inline int gtv_get_xscreen_num (GtkVideo *gtv)
 {
   return GDK_SCREEN_XNUMBER (gtv_get_screen (gtv));
 }
+#endif
 
 static inline gboolean is_fullscreen (gtk_video_private_t *priv)
 {
@@ -429,16 +445,11 @@ GtkType gtk_video_get_type (void)
 
 static void gtv_set_pointer (GtkVideo *gtv, gboolean state)
 {
-  Display *display = gtv_get_xdisplay (gtv);
   if (gtv->priv->pointer_visible != state || gtv->priv->pointer_shape != gtv->priv->pointer_shape_old)
   {
-    XLockDisplay (display);
     gtv->priv->pointer_visible = state;
     gtv->priv->pointer_shape_old = gtv->priv->pointer_shape;
-    XDefineCursor (display, gtv->priv->video_window,
-		   gtv->priv->pointer[state ? gtv->priv->pointer_shape + 1 : 0]);
-    XFlush (display);
-    XUnlockDisplay (display);
+    gdk_window_set_cursor(gtv->widget.window, gtv->priv->pointer[state ? gtv->priv->pointer_shape + 1 : 0]);
   }
 }
 
@@ -696,13 +707,14 @@ static gboolean end_time_check (gtk_video_private_t *priv)
 {
   gint pos_stream, pos_time, length_time;
   priv->end_time_id = 0;
+  xine_stream_t *const stream = priv->stream;
+
   if (!xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
     pos_stream = pos_time = length_time = 0;
   if (priv->end_time && pos_time >= priv->end_time)
   {
     logprintf ("gtkvideo: end time passed, sending stop event\n");
     priv->end_time = 0;
-    xine_stream_t *stream = priv->stream;
     xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
     xine_event_t event = {
       .type = XINE_EVENT_UI_PLAYBACK_FINISHED,
@@ -814,19 +826,18 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
   double                   res_h, res_v;
   int                      mm_h, mm_v, mm;
   const char              *video_driver_id;
-  xine_video_port_t       *video_port;
+  xine_video_port_t       *video_port = NULL;
   gtk_video_private_t     *priv = this->priv;
+  GdkScreen               *gdkscreen;
+  char                    *desc;
 
+#ifdef HAVE_X11
 #ifndef HAVE_XCB
   x11_visual_t vis;
 
   vis.display           = gtv_get_xdisplay (this);
   vis.screen            = gtv_get_xscreen_num (this);
   vis.d                 = priv->video_window;
-  mm_h			= DisplayWidthMM (vis.display, vis.screen);
-  res_h			= DisplayWidth (vis.display, vis.screen);
-  mm_v			= DisplayHeightMM (vis.display, vis.screen);
-  res_v			= DisplayHeight (vis.display, vis.screen);
 #else /* HAVE_XCB */
   xcb_visual_t vis;
   int screen_no = 0;
@@ -847,21 +858,49 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
   vis.connection        = connection;
   vis.screen            = screen_it.data;
   vis.window            = priv->video_window;
+  vis.dest_size_cb      = dest_size_cb;
+  vis.frame_output_cb   = frame_output_cb;
+  vis.user_data         = this;
 
 #ifdef ENABLE_X11_VO
   x11vis.display        = gtv_get_xdisplay (this);
   x11vis.screen         = gtv_get_xscreen_num (this);
   x11vis.d              = priv->video_window;
+  x11vis.dest_size_cb    = dest_size_cb;
+  x11vis.frame_output_cb = frame_output_cb;
+  x11vis.user_data       = this;
 #endif
-
-  /* why can't the 'Merkians spell "millimetre" correctly... */
-  mm_h			= screen_it.data->width_in_millimeters;
-  res_h			= screen_it.data->width_in_pixels;
-  mm_v			= screen_it.data->height_in_millimeters;
-  res_v			= screen_it.data->height_in_pixels;
 #endif /* HAVE_XCB */
 
-  char *desc = g_strdup_printf (_("The reported value is %dmm."), mm_h);
+#elif defined(WIN32) /* HAVE_X11 */
+  HWND desktop;
+  HDC hdc;
+  COLORREF colorkey;
+
+  memset(&priv->win32_visual, 0, sizeof priv->win32_visual);
+
+  priv->win32_visual.WndHnd = priv->video_window;
+  priv->win32_visual.HInst = GetModuleHandle(NULL);
+
+  desktop = GetDesktopWindow();
+  hdc = GetDC(desktop);
+  for(colorkey = 5; colorkey < 0xFF; colorkey++) {
+    if(colorkey == GetNearestColor(hdc, colorkey))
+      break;
+  }
+  priv->win32_visual.Brush = CreateSolidBrush(colorkey);
+  priv->win32_visual.ColorKey = (int)colorkey;
+  ReleaseDC(desktop, hdc);
+#endif
+
+
+  gdkscreen = gtv_get_screen(this);
+  mm_h = gdk_screen_get_width_mm(gdkscreen);
+  mm_v = gdk_screen_get_height_mm(gdkscreen);
+  res_h = gdk_screen_get_width(gdkscreen);
+  res_v = gdk_screen_get_height(gdkscreen);
+
+  desc = g_strdup_printf (_("The reported value is %dmm."), mm_h);
   mm = xine_config_register_range (priv->xine, "video.display_width",
 				   0, 0, 100000,
 				   N_("display width in mm\n(0 = autodetect)"), desc,
@@ -881,16 +920,6 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
   if (fabs(priv->display_ratio - 1.0) < 0.01)
     priv->display_ratio   = 1.0;
 
-  vis.dest_size_cb      = dest_size_cb;
-  vis.frame_output_cb   = frame_output_cb;
-  vis.user_data         = this;
-
-#ifdef ENABLE_X11_VO
-  x11vis.dest_size_cb    = dest_size_cb;
-  x11vis.frame_output_cb = frame_output_cb;
-  x11vis.user_data       = this;
-#endif
-
   char **choices = get_driver_ids (gxine_list_video_output_plugins (priv->xine));
   /* try to init video with stored information */
   int i = xine_config_register_enum (priv->xine,
@@ -899,6 +928,7 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
 				     NULL, 10, NULL, NULL);
   video_driver_id = priv->video_driver_id ? : choices[i];
 
+#ifdef HAVE_X11
   if (strcmp (video_driver_id, "auto"))
   {
     video_port = gxine_open_video_driver (priv->xine, video_driver_id,
@@ -912,13 +942,16 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
 
   video_port = gxine_open_video_driver (priv->xine, NULL,
 					(void *) &vis, (void *) &x11vis);
+#elif defined(WIN32)
+  video_port = xine_open_video_driver(priv->xine, video_driver_id, XINE_VISUAL_TYPE_DIRECTX, &priv->win32_visual);
+#endif
 
   if (video_port)
     return video_port;
 
   g_printerr (_("gtkvideo: video driver %s failed.\n"), "auto");
-  return gxine_open_video_driver (priv->xine, "none",
-				  (void *) &vis, (void *) &x11vis);
+  return xine_open_video_driver (priv->xine, "none", XINE_VISUAL_TYPE_NONE,
+                                 NULL);
 }
 
 uint32_t gtk_video_get_capabilities (GtkVideo *this)
@@ -954,6 +987,7 @@ gboolean gtk_video_in_spu_button (GtkVideo *gtv, int dir)
   return mkdtime (&now) - mkdtime (&priv->spu.time) <= 1.0;
 }
 
+#ifdef HAVE_X11
 static void gtv_send_xine_mouse_event (gtk_video_private_t *priv,
 				       int x, int y, int button)
 {
@@ -1114,12 +1148,16 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
 
   return GDK_FILTER_CONTINUE;
 }
+#endif
 
 static gboolean configure_cb (GtkWidget *widget, GdkEventConfigure *event,
 			      GtkVideo *this)
 {
   this->priv->pos.width = event->x + this->widget.allocation.width;
   this->priv->pos.height = event->y + this->widget.allocation.height;
+#if !defined(HAVE_X11) && defined(WIN32)
+  xine_port_send_gui_data (this->priv->video_port, GUI_WIN32_MOVED_OR_RESIZED, NULL);
+#endif
   return FALSE;
 }
 
@@ -1195,16 +1233,36 @@ static void gtv_atom_cardinal (Display *display, Window window,
 const GtkRequisition *gtk_video_get_fullscreen_geometry (GtkVideo *gtv)
 {
   GdkScreen *screen = gtv_get_screen (gtv);
+
+#if defined(HAVE_X11) && defined(HAVE_XINERAMA)
   Display *display = gtv_get_xdisplay (gtv);
 
-#ifdef HAVE_XINERAMA
   if (XineramaIsActive (display))
   {
+    GdkRectangle geom;
     int monitor = gdk_screen_get_monitor_at_window (screen, gtv->widget.window);
+    gdk_screen_get_monitor_geometry (screen, monitor, &geom);
+
     int count;
     XineramaScreenInfo *xinerama = XineramaQueryScreens (display, &count);
+
+    /* default first */
     gtv->priv->fullscreen.width = xinerama[monitor].width;
     gtv->priv->fullscreen.height = xinerama[monitor].height;
+
+    /* find the right one :-) */
+    for (monitor = 0; monitor < count; ++monitor)
+    {
+      if (xinerama[monitor].x_org  == geom.x &&
+          xinerama[monitor].y_org  == geom.y &&
+          xinerama[monitor].width  == geom.width &&
+          xinerama[monitor].height == geom.height)
+      {
+        gtv->priv->fullscreen.width = xinerama[monitor].width;
+        gtv->priv->fullscreen.height = xinerama[monitor].height;
+        break;
+      }
+    }
     XFree (xinerama);
   }
   else
@@ -1245,14 +1303,15 @@ static void gtk_video_realize (GtkWidget *widget)
   /* GdkWindowAttr attributes; */
   /* gint          attributes_mask; */
   GtkVideo            *this;
-  Pixmap               bm_no;
-  XColor               black_pixel;
   gtk_video_private_t *priv;
   GdkDisplay	      *display;
   GdkScreen	      *screen;
+#if defined(HAVE_X11) && !defined(USE_GDK_WINDOW)
   Display	      *xdisplay;
+  XColor               black_pixel;
 
   static char bm_no_data[] = { 0,0,0,0, 0,0,0,0 };
+#endif
 
   g_return_if_fail(widget != NULL);
   g_return_if_fail(GTK_IS_VIDEO(widget));
@@ -1270,11 +1329,12 @@ static void gtk_video_realize (GtkWidget *widget)
     return;
   }
 
+  screen = gdk_display_get_default_screen (display);
+
+#if defined(HAVE_X11) && !defined(USE_GDK_WINDOW)
   xdisplay = GDK_DISPLAY_XDISPLAY (display);
   XLockDisplay (xdisplay);
 
-  screen = gdk_display_get_default_screen (display);
-
   /*
    * create our own video window
    */
@@ -1295,6 +1355,30 @@ static void gtk_video_realize (GtkWidget *widget)
 
   widget->window = gdk_window_foreign_new_for_display
 		     (display, priv->video_window);
+#else /* HAVE_X11 && !USE_GDK_WINDOW */
+  GdkWindowAttr attrs = {
+       x: widget->allocation.x,
+       y: widget->allocation.y,
+       width: widget->allocation.width,
+       height: widget->allocation.height,
+       wclass: GDK_INPUT_OUTPUT,
+       window_type: GDK_WINDOW_CHILD,
+       type_hint: GDK_WINDOW_TYPE_HINT_NORMAL,
+       event_mask: GDK_ALL_EVENTS_MASK,
+  };
+  static GdkColor black = {};
+  GtkStyle *style;
+  int i;
+
+  memset(&black, 0, sizeof black);
+  widget->window = gdk_window_new(gtk_widget_get_parent_window(widget), &attrs, GDK_WA_X | GDK_WA_Y | GDK_WA_TYPE_HINT);
+#if defined(GDK_WINDOWING_X11)
+  priv->video_window = GDK_WINDOW_XWINDOW(widget->window);
+#elif defined(GDK_WINDOWING_WIN32)
+  priv->video_window = GDK_WINDOW_HWND(widget->window);
+#endif
+#endif /* HAVE_X11 && !USE_GDK_WINDOW */
+
   if (!widget->window)
   {
     g_printerr (_("gtkvideo: couldn't get a GDK handle for the video window!\n"));
@@ -1330,6 +1414,7 @@ static void gtk_video_realize (GtkWidget *widget)
 	"signal::screen-changed", G_CALLBACK (fs_reattach_screen_cb), NULL,
 	NULL);
 
+#ifdef HAVE_X11
 #ifdef HAVE_XTESTEXTENSION
   {
     int dummy1 = 0, dummy2 = 0, dummy3 = 0, dummy4 = 0;
@@ -1341,14 +1426,19 @@ static void gtk_video_realize (GtkWidget *widget)
 #endif
 
   gdk_window_set_events (widget->window, GDK_ALL_EVENTS_MASK);
+#ifndef USE_GDK_WINDOW
   XSelectInput (xdisplay, priv->video_window,
 		StructureNotifyMask | ExposureMask | FocusChangeMask |
 		KeyPressMask | KeyReleaseMask | PointerMotionMask |
 		ButtonPressMask | ButtonReleaseMask | ButtonMotionMask);
+#endif
   gdk_window_add_filter (widget->window, (GdkFilterFunc) xevent_filter_cb,
 			 widget);
 
+#ifndef USE_GDK_WINDOW
   XUnlockDisplay (xdisplay);
+#endif
+#endif /* HAVE_X11 */
 
   /*
    * load audio, video drivers
@@ -1366,22 +1456,17 @@ static void gtk_video_realize (GtkWidget *widget)
    * create mouse pointers
    */
 
-  XLockDisplay (xdisplay);
-  bm_no = XCreateBitmapFromData (xdisplay, priv->video_window,
-				 bm_no_data, 8, 8);
-  priv->pointer[0] = XCreatePixmapCursor (xdisplay, bm_no, bm_no,
-					  &black_pixel, &black_pixel, 0, 0);
-  priv->pointer[1] = XCreateFontCursor (xdisplay, XC_left_ptr);
-  priv->pointer[2] = XCreateFontCursor (xdisplay, XC_hand1);
+  priv->pointer[0] = gdk_cursor_new(GDK_BLANK_CURSOR);
+  priv->pointer[1] = gdk_cursor_new(GDK_LEFT_PTR);
+  priv->pointer[2] = gdk_cursor_new(GDK_HAND1);
   priv->pointer_visible = TRUE;
-  XUnlockDisplay (xdisplay);
 
 #ifdef XDG_SCREENSAVER_COMMAND
   priv->xdg_active = 0;
   priv->have_xdg_screensaver = !xdg_screensaver_poke ("--help", 0);
 #endif
   priv->screen_blanker_timeout =
-    g_timeout_add (4000, (GSourceFunc) gtv_screen_blanker_cb, gtv);
+    g_timeout_add (4000, (GSourceFunc) gtv_screen_blanker_cb, this);
 }
 
 
@@ -1408,10 +1493,6 @@ static void gtk_video_unrealize (GtkWidget *widget)
     priv->screen_blanker_timeout = 0;
   }
 
-  /* stop event thread */
-  if (priv->thread)
-    pthread_cancel (priv->thread);
-
   /* Hide all windows */
   if (GTK_WIDGET_MAPPED (widget))
     gtk_widget_unmap (widget);
@@ -1640,11 +1721,14 @@ void gtk_video_reshow (GtkVideo *gtv)
   }
 }
 
-void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
+void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen,
+			       GStaticRecMutex *const lock)
 {
   gtk_video_private_t  *priv;
   GtkWindow	       *toplevel;
+#ifdef HAVE_X11
   Display	       *display;
+#endif
 
   g_return_if_fail (gtv != NULL);
   g_return_if_fail (GTK_IS_VIDEO (gtv));
@@ -1654,9 +1738,11 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
   if (fullscreen == is_fullscreen (priv))
     return;
 
-  g_static_rec_mutex_lock (&engine_lock);
+  g_static_rec_mutex_lock (lock);
+#ifdef HAVE_X11
   display = gtv_get_xdisplay (gtv);
   XLockDisplay (display);
+#endif
   toplevel = GTK_WINDOW (gtk_widget_get_toplevel (&gtv->widget));
 
   if (fullscreen)
@@ -1698,6 +1784,7 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
 
     priv->fullscreen_mode = FALSE;
   }
+#ifdef HAVE_X11
 /* pointer motion will take care of this
   struct {
     Window root, child;
@@ -1710,7 +1797,8 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
 */
 
   XUnlockDisplay (display);
-  g_static_rec_mutex_unlock (&engine_lock);
+#endif
+  g_static_rec_mutex_unlock (lock);
 }
 
 gint gtk_video_is_fullscreen (GtkVideo *gtv)
@@ -2341,6 +2429,7 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
   else if (!dbus_deny)
     gtv_screensaver_maybe_inhibit_dbus (gtv->priv, dbus_deny = TRUE);
 
+#ifdef HAVE_X11
   Display *display = gtv_get_xdisplay (gtv);
   XLockDisplay (display);
 #ifdef HAVE_DPMS_EXTENSION
@@ -2365,6 +2454,7 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
   {
     XForceScreenSaver (display, ScreenSaverReset);
   }
+#endif /* HAVE_X11 */
 
   if (!screensaver_is_running_dbus (gtv->priv))
   {
@@ -2394,7 +2484,9 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
 #endif
   }
 
+#ifdef HAVE_X11
   XUnlockDisplay (display);
+#endif
 
   return FALSE;
 }
