@@ -28,7 +28,6 @@
 
 #include <glib.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <xine.h>
@@ -264,7 +263,7 @@ gboolean buttonpress_cb (GtkWidget *win, GdkEventButton *event, gpointer data)
     {
     /* Full-screen toggle */
     case 1:
-      if (event->type == GDK_2BUTTON_PRESS && !gtk_video_in_spu_button (gtv, -1))
+      if (event->type == GDK_2BUTTON_PRESS && !gtk_video_in_spu_button ((GtkVideo *)gtv, -1))
 	gtk_action_activate (GTK_ACTION (action_items.fullscreen));
       return TRUE;
     /* Toolbar toggle (middle button) */
@@ -690,9 +689,9 @@ static void load_key_bindings (void)
   if (kbfile)
   {
     xml_node_t *root = NULL;
-    xml_parser_init (kbfile, strlen (kbfile), XML_PARSER_CASE_INSENSITIVE);
+    xml_parser_init_R (xml_parser_t *xml, kbfile, strlen (kbfile), XML_PARSER_CASE_INSENSITIVE);
 
-    if (xml_parser_build_tree (&root) >= 0)
+    if (xml_parser_build_tree_R (xml, &root) >= 0)
     {
       if (!strcasecmp (root->name, "gxinekb"))
       {
@@ -740,6 +739,7 @@ static void load_key_bindings (void)
       load_default_kb();
     }
 
+    xml_parser_finalize_R (xml);
     free (kbfile);
   }
   else
@@ -1041,10 +1041,13 @@ static JSBool js_xine_event (int type
 #endif
 
 #define JS_XINE_EVENT(L,T) \
-  static JSBool js_##L (JSContext *cx, JSObject *obj, uintN argc, \
-			jsval *argv, jsval *rval) \
+  static JSBool js_##L (JSContext *cx, uintN argc, jsval *vp) \
   { \
-    return js_xine_event (XINE_EVENT_##T, #L); \
+    JSBool ret = js_xine_event (XINE_EVENT_##T, #L); \
+    if (ret) { \
+      JS_SET_RVAL (cx, vp, JSVAL_VOID); \
+    } \
+    return ret; \
   }
 
 JS_XINE_EVENT (input_up, INPUT_UP);
@@ -1091,16 +1094,15 @@ JS_XINE_EVENT (input_menu3, INPUT_MENU3);
 JS_XINE_EVENT (input_previous, INPUT_PREVIOUS);
 JS_XINE_EVENT (input_next, INPUT_NEXT);
 
-static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc,
-				   jsval *argv, jsval *rval)
+static JSBool js_keybindings_show (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("keybindings_show");
   kb_edit_show ();
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_keypad_show (JSContext *cx, JSObject *obj, uintN argc,
-			      jsval *argv, jsval *rval)
+static JSBool js_keypad_show (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("keypad_show");
   if (keypad)
@@ -1110,6 +1112,7 @@ static JSBool js_keypad_show (JSContext *cx, JSObject *obj, uintN argc,
     else
       window_show (keypad, NULL);
   }
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -1231,6 +1234,7 @@ static const kb_xine_event_map_t xine_vdr = {
     { "STOP", XINE_EVENT_VDR_STOP },
     { "SUBTITLES", XINE_EVENT_VDR_INFO + 2 /* ..._VDR_SUBTITLES */ },
     { "TIMERS", XINE_EVENT_VDR_TIMERS },
+    { "USER0", XINE_EVENT_VDR_INFO + 3 /* ..._VDR_USER0 */ },
     { "USER1", XINE_EVENT_VDR_USER1 },
     { "USER2", XINE_EVENT_VDR_USER2 },
     { "USER3", XINE_EVENT_VDR_USER3 },
@@ -1247,8 +1251,8 @@ static const kb_xine_event_map_t xine_vdr = {
   }
 };
 
-JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-			 jsval *rval, const char *func, const char *const *prefix,
+JSBool js_event_generic (JSContext *cx, uintN argc, jsval *vp,
+                         const char *func, const char *const *prefix,
 			 ...)
 {
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
@@ -1257,8 +1261,9 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
   char *evstr;
   va_list ap;
   const kb_xine_event_map_t *evlist;
+  jsval *argv = JS_ARGV (cx, vp);
 
-  *rval = JSVAL_TRUE;
+  JS_SET_RVAL (cx, vp, JSVAL_TRUE);
 
   se_log_fncall (func);
   se_argc_check_max (1, func);
@@ -1275,7 +1280,7 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 
   se_arg_is_string (0, func);
 
-  *rval = JSVAL_FALSE;
+  JS_SET_RVAL (cx, vp, JSVAL_FALSE);
 
   if (prefix)
   {
@@ -1294,7 +1299,7 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
   }
 
   str = JS_ValueToString (cx, argv[0]);
-  evstr = JS_GetStringBytes (str);
+  evstr = SE_JS_ENCODE_STRING (cx, str);
   event.type = 0;
 
   va_start (ap, prefix);
@@ -1306,8 +1311,10 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     event.type = kb_xine_event_lookup (NULL, evstr);
 
   if (!event.type)
+  {
+    SE_JS_FREE_ENCODED_STRING (cx, evstr);
     return JS_TRUE;
-
+  }
   if (event.type != -1)
   {
     event.data = NULL;
@@ -1316,21 +1323,20 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     xine_event_send (stream, &event);
   }
 
-  *rval = JSVAL_TRUE;
+  SE_JS_FREE_ENCODED_STRING (cx, evstr);
+  JS_SET_RVAL (cx, vp, JSVAL_TRUE);
   return JS_TRUE;
 }
 
-static JSBool js_event (JSContext *cx, JSObject *obj, uintN argc,
-		      jsval *argv, jsval *rval)
+static JSBool js_event (JSContext *cx, uintN argc, jsval *vp)
 {
-  return js_event_generic (cx, obj, argc, argv, rval, "event", NULL, NULL);
+  return js_event_generic (cx, argc, vp, "event", NULL, NULL);
 }
 
-static JSBool js_vdr (JSContext *cx, JSObject *obj, uintN argc,
-		      jsval *argv, jsval *rval)
+static JSBool js_vdr (JSContext *cx, uintN argc, jsval *vp)
 {
   const char *const prefixes[] = { "vdr:/", "netvdr:/", NULL };
-  return js_event_generic (cx, obj, argc, argv, rval, "vdr", prefixes,
+  return js_event_generic (cx, argc, vp, "vdr", prefixes,
 			   &xine_vdr, NULL);
 }
 
@@ -1528,25 +1534,25 @@ void key_events_init (void)
 
   {
     static const se_f_def_t defs[] = {
-      { "input_up", js_input_up, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_down", js_input_down, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_left", js_input_left, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_right", js_input_right, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_select", js_input_select, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_menu", js_input_menu, 0, 0,
+      { "input_up", js_input_up, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_down", js_input_down, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_left", js_input_left, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_right", js_input_right, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_select", js_input_select, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_menu", js_input_menu, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_INPUT, N_("int"), N_("range is 1 to 7") },
-      { "input_menu1", js_input_menu1, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_menu2", js_input_menu2, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_menu3", js_input_menu3, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_previous", js_input_previous, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "input_next", js_input_next, 0, 0, SE_GROUP_INPUT, NULL, NULL },
-      { "keybindings_show", js_keybindings_show, 0, 0,
+      { "input_menu1", js_input_menu1, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_menu2", js_input_menu2, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_menu3", js_input_menu3, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_previous", js_input_previous, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "input_next", js_input_next, 0, JSFUN_FAST_NATIVE, SE_GROUP_INPUT, NULL, NULL },
+      { "keybindings_show", js_keybindings_show, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_DIALOGUE, NULL, NULL },
-      { "keypad_show", js_keypad_show, 0, 0,
+      { "keypad_show", js_keypad_show, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_DIALOGUE, NULL, NULL },
-      { "event", js_event, 0, 0,
+      { "event", js_event, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_INPUT, N_("string"), N_("event; returns true if sent") },
-      { "vdr", js_vdr, 0, 0,
+      { "vdr", js_vdr, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_EXTERNAL, N_("string"), N_("VDR event; returns true if sent") },
       { NULL }
     };
