@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2001-2003 the xine project
+/*
+ * Copyright (C) 2001-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,17 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: lirc.h,v 1.2 2006-02-21 19:06:12 dsalt Exp $
- *
  * lirc handling functions
  */
 
-#ifndef HAVE_LIRC_H
-#define HAVE_LIRC_H
-
-void gxine_lirc_init (void) ;
-
-void gxine_lirc_quit (void) ;
+#ifndef GXINE_LIRC_H
+#define GXINE_LIRC_H
 
+void gxine_lirc_init (void);
+void gxine_lirc_quit (void);
 
 #endif
