@@ -269,7 +269,7 @@ JS_CB_T (Fullscreen)
 {
   if (!GTK_WIDGET_MAPPED (app))
     app_show ();
-  gtk_video_set_fullscreen ((GtkVideo *)gtv, v);
+  gtk_video_set_fullscreen ((GtkVideo *)gtv, v, &engine_lock);
   gtk_action_set_sensitive (action_items.hidemain, !v);
   gtk_action_set_sensitive (action_items.showmain, !v);
   if (v)
