@@ -34,8 +34,6 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#include <X11/Xlib.h>
-
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
@@ -432,9 +430,9 @@ static int playlist_load (const char *fname)
     goto ret0;
   }
 
-  xml_parser_init (plfile, st.st_size, XML_PARSER_CASE_INSENSITIVE);
+  xml_parser_init_R (xml_parser_t *xml, plfile, st.st_size, XML_PARSER_CASE_INSENSITIVE);
 
-  if (xml_parser_build_tree (&root) < 0)
+  if (xml_parser_build_tree_R (xml, &root) < 0)
   {
     display_error (FROM_GXINE, _("Loading of playlist file failed."),
 		   _("‘%s’ is not a valid XML/ASX file"), fname);
@@ -528,6 +526,7 @@ static int playlist_load (const char *fname)
   ret0:
   free (plfile);
   xml_parser_free_tree (root);
+  xml_parser_finalize_R (xml);
   if (defaultfile)
     free ((char *)fname);
   return ret;
@@ -786,11 +785,11 @@ typedef struct {
   union {
     struct {
       gboolean (*parse) (const char *const *, void **);
-      play_item_t *(*fetch) (const char *const *, size_t, void *);
+      play_item_t *(*fetch) (const char *const *, size_t, void *, const char *);
     } text;
     struct {
       gboolean (*parse) (pl_xml_info_t *, void **);
-      play_item_t *(*fetch) (pl_xml_info_t *, void *);
+      play_item_t *(*fetch) (pl_xml_info_t *, void *, const char *);
       const char *root;
     } xml;
   };
@@ -823,6 +822,9 @@ playlist_load_generic (const char *mrl, gint ins_pos,
   xml_node_t *xml_tree;
   pl_xml_info_t *xml_info = NULL;
   void *data = NULL;
+#ifdef HAVE_XML_PARSER_REENTRANT
+  xml_parser_t *xml = NULL;
+#endif
 
   if (ins_pos == -2)
   {
@@ -837,15 +839,17 @@ playlist_load_generic (const char *mrl, gint ins_pos,
 
   if (loader->is_xml)
   {
-    xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-    if (xml_parser_build_tree (&xml_tree))
+    xml_parser_init_R (xml, contents, size, XML_PARSER_CASE_INSENSITIVE);
+    if (xml_parser_build_tree_R (xml, &xml_tree))
     {
+      xml_parser_finalize_R (xml);
       g_free (contents);
       return -1;
     }
     if (loader->xml.root && strcmp (xml_tree->name, loader->xml.root))
     {
       g_printerr (_("playlist: no %s tag\n"), loader->xml.root);
+      xml_parser_finalize_R (xml);
       g_free (contents);
       return -1;
     }
@@ -875,9 +879,9 @@ playlist_load_generic (const char *mrl, gint ins_pos,
   {
     play_item_t *item;
     while ((item = loader->is_xml
-		   ? loader->xml.fetch (xml_info, data)
+		   ? loader->xml.fetch (xml_info, data, mrl)
 		   : loader->text.fetch ((const char *const *)lines,
-					 index, data)))
+					 index, data, mrl)))
     {
       if (item == (play_item_t *)-1)
       {
@@ -904,6 +908,7 @@ playlist_load_generic (const char *mrl, gint ins_pos,
   if (loader->is_xml)
   {
     xml_parser_free_tree (xml_tree);
+    xml_parser_finalize_R (xml);
     free (xml_info);
   }
   else
@@ -915,7 +920,7 @@ playlist_load_generic (const char *mrl, gint ins_pos,
 /* Load SMIL playlists */
 
 static play_item_t *
-pl_smil_fetch (pl_xml_info_t *info, void *data)
+pl_smil_fetch (pl_xml_info_t *info, void *data, const char *mrl)
 {
   for (;;)
   {
@@ -1008,7 +1013,7 @@ pl_ram_download (const char *mrl, ssize_t *size_p)
 }
 
 static play_item_t *
-pl_ram_fetch (const char *const *lines, size_t index, void *data)
+pl_ram_fetch (const char *const *lines, size_t index, void *data, const char *mrl)
 {
   /* ignore comments */
   if (lines[index][0] == '#')
@@ -1043,10 +1048,18 @@ playlist_load_ram (const char *mrl, gint ins_pos)
 /* Load M3U playlists */
 
 static play_item_t *
-pl_m3u_fetch (const char *const *lines, size_t index, void *data)
+pl_m3u_fetch (const char *const *lines, size_t index, void *data, const char *mrl)
 {
-  /* Either it's a URI, or it has a proper path */
-  if (!strstr (lines[index], "://") && lines[index][0] != G_DIR_SEPARATOR)
+  /* Ignore comments */
+  if (lines[index][0] == '#')
+    return NULL;
+
+  /* Ignore blank lines */
+  int i;
+  for (i = strlen (lines[index]) - 1; i >= 0; --i)
+    if (!isspace (lines[index][i]))
+      break;
+  if (i < 0)
     return NULL;
 
   /* is this type parsing really needed? */
@@ -1056,7 +1069,17 @@ pl_m3u_fetch (const char *const *lines, size_t index, void *data)
 		       : (extension && !strncasecmp (extension, ".ogg", 4))
 		       ? "#demux:ogg"
 		       : NULL;
-  char *tmp = g_strconcat (lines[index], suffix, NULL);
+  char *tmp = NULL;
+  /* Relative path? */
+  if (!strstr (lines[index], "://") && lines[index][0] != G_DIR_SEPARATOR)
+  {
+    const char *slash = strrchr (mrl, '/');
+    if (slash)
+      tmp = g_strdup_printf ("%.*s%s%s", (int)(slash + 1 - mrl), mrl, lines[index], suffix);
+  }
+  if (!tmp)
+    tmp = g_strconcat (lines[index], suffix, NULL);
+
   play_item_t *item = play_item_new (NULL, tmp, 0, 0);
   free (tmp);
 
@@ -1111,7 +1134,7 @@ pl_pls_parse (const char *const *lines, void **data)
 }
 
 static play_item_t *
-pl_pls_fetch (const char *const *lines, size_t index, void *data)
+pl_pls_fetch (const char *const *lines, size_t index, void *data, const char *mrl)
 {
   if (++index > *(unsigned int *)data)
     return NULL;
@@ -1177,8 +1200,8 @@ static int playlist_load_asx (const char *mrl, gint ins_pos)
   if (! (contents = read_entire_file (mrl, &size)))
     return ret;
 
-  xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-  res = xml_parser_build_tree (&xml_tree);
+  xml_parser_init_R (xml_parser_t *xml, contents, size, XML_PARSER_CASE_INSENSITIVE);
+  res = xml_parser_build_tree_R (xml, &xml_tree);
 
   if (res)
   {
@@ -1200,6 +1223,7 @@ static int playlist_load_asx (const char *mrl, gint ins_pos)
 	  ret = playlist_add_mrl (strstr(lines[i], "mms://"), ins_pos);
       }
     }
+    xml_parser_finalize_R (xml);
     return ret;
   }
 
@@ -1343,6 +1367,7 @@ static int playlist_load_asx (const char *mrl, gint ins_pos)
     g_printerr(_("playlist: no ASX tag\n"));
 
   xml_parser_free_tree (xml_tree);
+  xml_parser_finalize_R (xml);
   g_free (contents);
 
   return ret;
@@ -1824,6 +1849,7 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
       xine_spu_button_t *btn = event->data;
       gtk_video_in_spu_button (gtv, btn->direction);
     }
+    break;
 
 #ifndef XINE_EVENT_MRL_REFERENCE_EXT
 /* present in 1.1.0 but not 1.0.2 */
@@ -2123,71 +2149,70 @@ static void drop_cb (GtkTreeView	*widget,
  * js functions
  */
 
-static JSBool js_playlist_get_item (JSContext *cx, JSObject *obj, uintN argc,
-				    jsval *argv, jsval *rval)
+static JSBool js_playlist_get_item (JSContext *cx, uintN argc, jsval *vp)
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   se_log_fncall ("playlist_get_item");
-  *rval = INT_TO_JSVAL (playlist_get_list_pos());
+  JS_SET_RVAL (cx, vp, INT_TO_JSVAL (playlist_get_list_pos()));
   return JS_TRUE;
 }
 
-static JSBool js_playlist_clear (JSContext *cx, JSObject *obj, uintN argc,
-				 jsval *argv, jsval *rval)
+static JSBool js_playlist_clear (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("playlist_clear");
   playlist_clear ();
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_playlist_flush (JSContext *cx, JSObject *obj, uintN argc,
-				 jsval *argv, jsval *rval)
+static JSBool js_playlist_flush (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("playlist_flush");
   playlist_flush (PLAY_ITEM_NORMAL);
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_playlist_load (JSContext *cx, JSObject *obj, uintN argc,
-				jsval *argv, jsval *rval)
+static JSBool js_playlist_load (JSContext *cx, uintN argc, jsval *vp)
 {
+  jsval *argv = JS_ARGV (cx, vp);
   se_log_fncall_checkinit ("playlist_load");
 
   se_argc_check (1, "playlist_load");
   se_arg_is_string_or_null (0, "playlist_load");
 
-  char *mrl = JS_GetStringBytes (JS_ValueToString (cx, argv[0]));
+  char *mrl = SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[0]));
 
   logprintf ("playlist_load: file=%s\n", mrl);
   playlist_load_any (mrl);
 
-  *rval = JSVAL_VOID;
+  SE_JS_FREE_ENCODED_STRING (cx, mrl);
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_playlist_add (JSContext *cx, JSObject *obj, uintN argc,
-			       jsval *argv, jsval *rval)
+static JSBool js_playlist_add (JSContext *cx, uintN argc, jsval *vp)
 {
+  jsval *argv = JS_ARGV (cx, vp);
 
   se_log_fncall_checkinit ("playlist_add");
 
   se_argc_check_range (1, 2, "playlist_add");
   se_arg_is_string (0, "playlist_add");
 
-  char *mrl = JS_GetStringBytes (JS_ValueToString (cx, argv[0]));
+  char *mrl = SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[0]));
   int item;
 
   if (argc > 1)
   {
     se_arg_is_string_or_null (1, "playlist_add");
     char *title = JSVAL_IS_STRING (argv[1])
-		  ? JS_GetStringBytes (JS_ValueToString (cx, argv[1]))
+                  ? SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[1]))
 		  : NULL;
     logprintf ("playlist_add: MRL=%s title=%s\n", mrl, title);
     play_item_t *play_item = play_item_new (title, mrl, 0, 0);
     item = playlist_add (play_item, -1);
+    SE_JS_FREE_ENCODED_STRING (cx, title);
   }
   else
   {
@@ -2195,14 +2220,15 @@ static JSBool js_playlist_add (JSContext *cx, JSObject *obj, uintN argc,
     item = playlist_add_mrl (mrl, -1);
   }
 
-  *rval = INT_TO_JSVAL (item);
+  SE_JS_FREE_ENCODED_STRING (cx, mrl);
+  JS_SET_RVAL (cx, vp, INT_TO_JSVAL (item));
 
   return JS_TRUE;
 }
 
-static JSBool js_playlist_delete (JSContext *cx, JSObject *obj, uintN argc,
-				  jsval *argv, jsval *rval)
+static JSBool js_playlist_delete (JSContext *cx, uintN argc, jsval *vp)
 {
+  jsval *argv = JS_ARGV (cx, vp);
   se_log_fncall_checkinit ("playlist_remove");
 
   se_argc_check_range (1, 2, "playlist_remove");
@@ -2227,14 +2253,14 @@ static JSBool js_playlist_delete (JSContext *cx, JSObject *obj, uintN argc,
     gtk_list_store_remove (pl_store, &iter);
   }
 
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_playlist_play (JSContext *cx, JSObject *obj, uintN argc,
-				jsval *argv, jsval *rval)
+static JSBool js_playlist_play (JSContext *cx, uintN argc, jsval *vp)
 {
   int32 item;
+  jsval *argv = JS_ARGV (cx, vp);
 
   se_log_fncall_checkinit ("playlist_play");
 
@@ -2245,21 +2271,19 @@ static JSBool js_playlist_play (JSContext *cx, JSObject *obj, uintN argc,
 
   playlist_play (item);
 
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_playlist_show (JSContext *cx, JSObject *obj, uintN argc,
-				jsval *argv, jsval *rval)
+static JSBool js_playlist_show (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("playlist_show");
   playlist_show ();
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
-static JSBool js_mrl_browser_refresh (JSContext *cx, JSObject *obj, uintN argc,
-				      jsval *argv, jsval *rval)
+static JSBool js_mrl_browser_refresh (JSContext *cx, uintN argc, jsval *vp)
 {
   GtkTreeIter iter;
   se_log_fncall_checkinit ("mrl_browser_refresh");
@@ -2270,14 +2294,14 @@ static JSBool js_mrl_browser_refresh (JSContext *cx, JSObject *obj, uintN argc,
       || !item_marked_current (&iter))
   {
     playlist_flush (PLAY_ITEM_BROWSER);
-    *rval = JSVAL_VOID;
+    JS_SET_RVAL (cx, vp, JSVAL_VOID);
     return JS_TRUE;
   }
 
   CUR_ITEM_LOCK ();
   playlist_browse_set (cur_item ? : playlist_get_item (cur_list_pos));
   CUR_ITEM_UNLOCK ();
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -2604,23 +2628,23 @@ void playlist_init (void)
 
   {
     static const se_f_def_t defs[] = {
-      { "playlist_show", js_playlist_show, 0, 0,
+      { "playlist_show", js_playlist_show, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_DIALOGUE, NULL, NULL },
-      { "playlist_clear", js_playlist_clear, 0, 0,
+      { "playlist_clear", js_playlist_clear, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, NULL, NULL },
-      { "playlist_flush", js_playlist_flush, 0, 0,
+      { "playlist_flush", js_playlist_flush, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, NULL, NULL },
-      { "playlist_load", js_playlist_load, 0, 0,
+      { "playlist_load", js_playlist_load, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, N_("file"), NULL },
-      { "playlist_add", js_playlist_add, 0, 0,
+      { "playlist_add", js_playlist_add, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, N_("MRL[, title]"), NULL },
-      { "playlist_delete", js_playlist_delete, 0, 0,
+      { "playlist_delete", js_playlist_delete, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, N_("int"), NULL },
-      { "playlist_play", js_playlist_play, 0, 0,
+      { "playlist_play", js_playlist_play, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, N_("int"), N_("playlist entry number") },
-      { "playlist_get_item", js_playlist_get_item, 0, 0,
+      { "playlist_get_item", js_playlist_get_item, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_PLAYLIST, NULL, NULL },
-      { "mrl_browser_refresh", js_mrl_browser_refresh, 0, 0,
+      { "mrl_browser_refresh", js_mrl_browser_refresh, 0, JSFUN_FAST_NATIVE,
 	SE_GROUP_HIDDEN, NULL, NULL },
       { NULL }
     };
