@@ -25,7 +25,6 @@
 #include "globals.h"
 
 #include <assert.h>
-#include <X11/Xlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
@@ -1106,12 +1105,12 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
-static JSBool js_preferences_show (JSContext *cx, JSObject *obj, uintN argc,
-				   jsval *argv, jsval *rval)
+static JSBool js_preferences_show (JSContext *cx, uintN argc, jsval *vp)
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   se_log_fncall_checkinit ("preferences_show");
   preferences_show ();
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -1171,7 +1170,7 @@ void preferences_init (void)
 {
   is_visible = FALSE;
   /* script engine functions */
-  se_defun (gse, NULL, "preferences_show", js_preferences_show, 0, 0,
+  se_defun (gse, NULL, "preferences_show", js_preferences_show, 0, JSFUN_FAST_NATIVE,
 	    SE_GROUP_DIALOGUE, NULL, NULL);
 
   preferences_init_dbox ();
