@@ -34,7 +34,6 @@
 #include <stdarg.h>
 
 #include <gdk/gdkkeysyms.h>
-#include <jsstr.h>
 
 #include "script_engine.h"
 #include "ui.h"
@@ -53,6 +52,7 @@
 #include "snapshot.h"
 #include "engine.h"
 #include "history.h"
+
 /*
 #define LOG
 */
@@ -194,7 +194,10 @@ gchar *se_result_str (se_t *se)
     return NULL;
 
   se->str = JS_ValueToString (se->cx, se->rval);
-  return JS_GetStringBytes (se->str);
+  char *encoded = SE_JS_ENCODE_STRING(se->cx, se->str);
+  char *res = strdup(encoded);
+  SE_JS_FREE_ENCODED_STRING(se->cx, encoded);
+  return res;
 }
 
 int se_result_int (se_t *se, JSInt32 *num)
@@ -253,8 +256,7 @@ se_js_string_val (JSContext *cx, const char *str)
  * methods
  */
 
-static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc,
-			     jsval *argv, jsval *rval)
+static JSBool controls_exit (JSContext *cx, uintN argc, jsval *vp)
 {
   gchar *fname;
 
@@ -299,8 +301,7 @@ static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc,
   exit (0);
 }
 
-static JSBool show_js_console (JSContext *cx, JSObject *obj, uintN argc,
-			       jsval *argv, jsval *rval)
+static JSBool show_js_console (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("show_js_console");
   if (GTK_WIDGET_VISIBLE (se_window))
@@ -311,85 +312,93 @@ static JSBool show_js_console (JSContext *cx, JSObject *obj, uintN argc,
     se_prop_set_int (gse, js_icon_obj, "v", 0);
   }
 
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
 static JSBool
-js_callback (JSContext *cx, JSObject *obj,
-	     uintN argc, jsval *argv, jsval *rval)
+js_callback (JSContext *cx, uintN argc, jsval *vp)
 {
+  jsval *argv = JS_ARGV (cx, vp);
   se_t *se = (se_t *) JS_GetContextPrivate (cx);
   se_log_fncall ("callback");
   se_argc_check_range (1, 2, "callback");
   se_arg_is_string (0, "callback");
   if (argc > 1)
     se_arg_is_object (1, "callback");
-  engine_queue_push (JS_GetStringBytes (JS_ValueToString (cx, argv[0])),
-		     JS_GetPrivate (cx, argc > 1 ? JSVAL_TO_OBJECT (argv[1])
-						 : se->global),
-		     NULL, NULL, NULL, _("JS callback"));
-  *rval = JSVAL_VOID;
+  char *str = SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[0]));
+  engine_queue_push (str,
+ 		     JS_GetPrivate (cx, argc > 1 ? JSVAL_TO_OBJECT (argv[1])
+ 						 : se->global),
+ 		     NULL, NULL, NULL, _("JS callback"));
+  SE_JS_FREE_ENCODED_STRING (cx, str);
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
 static JSBool
-js_xine_cfg_get (JSContext *cx, JSObject *obj,
-		 uintN argc, jsval *argv, jsval *rval)
+js_xine_cfg_get (JSContext *cx, uintN argc, jsval *vp)
 {
+  jsval *argv = JS_ARGV (cx, vp);
   se_log_fncall ("xine_cfg_get");
   se_argc_check (1, "xine_cfg_get");
   se_arg_is_string (0, "xine_cfg_get");
 
   xine_cfg_entry_t entry;
-  char *cfg = JS_GetStringBytes (JS_ValueToString (cx, argv[0]));
+  char *cfg = SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[0]));
   if (!xine_config_lookup_entry (xine, cfg, &entry))
   {
-    *rval = JSVAL_NULL;
+    JS_SET_RVAL (cx, vp, JSVAL_NULL);
+    SE_JS_FREE_ENCODED_STRING (cx, cfg);
     return JS_TRUE;
   }
 
+  SE_JS_FREE_ENCODED_STRING(cx, cfg);
+
   switch (entry.type)
   {
   case XINE_CONFIG_TYPE_RANGE:
   case XINE_CONFIG_TYPE_NUM:
-    *rval = INT_TO_JSVAL (entry.num_value);
+    JS_SET_RVAL (cx, vp, INT_TO_JSVAL (entry.num_value));
     return JS_TRUE;
 
   case XINE_CONFIG_TYPE_BOOL:
-    *rval = BOOLEAN_TO_JSVAL (entry.num_value);
+    JS_SET_RVAL (cx, vp, BOOLEAN_TO_JSVAL (entry.num_value));
     return JS_TRUE;
 
   case XINE_CONFIG_TYPE_STRING:
-    *rval = se_js_string_val (cx, entry.str_value ? : entry.str_default);
+    JS_SET_RVAL (cx, vp, se_js_string_val (cx, entry.str_value ? : entry.str_default));
     return JS_TRUE;
 
   case XINE_CONFIG_TYPE_ENUM:
-    *rval = se_js_string_val (cx, entry.enum_values[entry.num_value]);
+    JS_SET_RVAL (cx, vp, se_js_string_val (cx, entry.enum_values[entry.num_value]));
     return JS_TRUE;
 
   default:
-    *rval = JSVAL_NULL;
+    JS_SET_RVAL (cx, vp, JSVAL_NULL);
     return JS_TRUE;
   }
 }
 
 static JSBool
-js_xine_cfg_set (JSContext *cx, JSObject *obj,
-		 uintN argc, jsval *argv, jsval *rval)
+js_xine_cfg_set (JSContext *cx, uintN argc, jsval *vp)
 {
+  jsval *argv = JS_ARGV (cx, vp);
   se_log_fncall ("xine_cfg_set");
   se_argc_check (2, "xine_cfg_set");
   se_arg_is_string (0, "xine_cfg_set");
 
   xine_cfg_entry_t entry;
-  char *cfg = JS_GetStringBytes (JS_ValueToString (cx, argv[0]));
+  char *cfg = SE_JS_ENCODE_STRING (cx, JS_ValueToString(cx, argv[0]));
   if (!xine_config_lookup_entry (xine, cfg, &entry))
   {
-    *rval = JSVAL_VOID;
+    JS_SET_RVAL (cx, vp, JSVAL_VOID);
+    SE_JS_FREE_ENCODED_STRING (cx, cfg);
     return JS_TRUE;
   }
 
+  SE_JS_FREE_ENCODED_STRING (cx, cfg);
+
   int32 num;
   switch (entry.type)
   {
@@ -413,28 +422,34 @@ js_xine_cfg_set (JSContext *cx, JSObject *obj,
 
   case XINE_CONFIG_TYPE_STRING:
     se_arg_is_string (1, "xine_cfg_set");
-    entry.str_value = JS_GetStringBytes (JS_ValueToString (cx, argv[1]));
+    entry.str_value = SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[1]));
     break;
 
   case XINE_CONFIG_TYPE_ENUM:
     se_arg_is_string (1, "xine_cfg_set");
-    char *v = JS_GetStringBytes (JS_ValueToString (cx, argv[1]));
+    char *v = SE_JS_ENCODE_STRING (cx, JS_ValueToString (cx, argv[1]));
     int i;
     for (i = 0; entry.enum_values[i]; ++i)
-      if (!strcmp (v, entry.enum_values[i]))
+      if (!strcmp (v, entry.enum_values[i])) {
+        SE_JS_FREE_ENCODED_STRING (cx, v);
 	break;
+      }
     if (entry.enum_values[i])
       entry.num_value = i;
+    SE_JS_FREE_ENCODED_STRING (cx, v);
     break;
 
   default:
-    *rval = JSVAL_VOID;
+    JS_SET_RVAL (cx, vp, JSVAL_VOID);
     return JS_TRUE;
   }
 
   preferences_update_entry (&entry);
 
-  *rval = JSVAL_VOID;
+  if (entry.type == XINE_CONFIG_TYPE_STRING)
+    SE_JS_FREE_ENCODED_STRING (cx, entry.str_value);
+
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -476,8 +491,7 @@ static char *show_help_int (se_t *se, se_o_t *obj, se_group_t selector)
   return help;
 }
 
-static JSBool show_help (JSContext *cx, JSObject *obj, uintN argc,
-			  jsval *argv, jsval *rval)
+static JSBool show_help (JSContext *cx, uintN argc, jsval *vp)
 {
   static char *const group_id[] = {
     NULL,
@@ -530,7 +544,7 @@ static JSBool show_help (JSContext *cx, JSObject *obj, uintN argc,
    * int get_time ();
    * int get_length ();
    */
-  *rval = JSVAL_VOID;
+  JS_SET_RVAL (cx, vp, JSVAL_VOID);
   return JS_TRUE;
 }
 
@@ -652,8 +666,9 @@ static jsval get_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p)
 
   case SE_TYPE_DOUBLE:
     {
-      jsdouble *d = JS_NewDouble (se->cx, get_prop_double (se, o, p));
-      return DOUBLE_TO_JSVAL (d);
+      jsval rval;
+      SE_JS_NEW_NUMBER_VAL (se->cx, get_prop_double (se, o, p), rval);
+      return rval;
     }
 
   case SE_TYPE_BOOL:
@@ -692,7 +707,7 @@ static void set_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p, const jsval *v)
 
   case SE_TYPE_DOUBLE:
     if (JSVAL_IS_NUMBER(*v))
-      se_prop_set_double (se, o, p->id, *JSVAL_TO_DOUBLE(*v));
+      se_prop_set_double (se, o, p->id, SE_JSVAL_TO_DOUBLE(*v));
     else
       se->print_cb (se->print_cb_data, _("\n%s.%s: value must be numeric\n"),
 		    o->id, p->id);
@@ -709,8 +724,9 @@ static void set_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p, const jsval *v)
   case SE_TYPE_STRING:
     {
       JSString *str = JSVAL_TO_STRING(*v);
-      char     *string = JS_GetStringBytes (str);
+      char     *string = SE_JS_ENCODE_STRING (se->cx, str);
       se_prop_set (se, o, p->id, string);
+      SE_JS_FREE_ENCODED_STRING (se->cx, str);
     }
     return;
 
@@ -725,17 +741,17 @@ static void set_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p, const jsval *v)
  * function to create and maintain js objects
  */
 
-static JSBool
-generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+SE_JS_PROPERTY_GETTER_DECLARE (generic_JSGetProperty)
 {
+  SE_JS_PROPERTY_GETTER_INIT_VARIABLES (id_val);
   se_t   *se = (se_t *) JS_GetContextPrivate(cx);
   se_log_fncall ("generic get property");
   se_o_t *o = JS_GetPrivate (cx, obj);
 
-  if (JSVAL_IS_STRING (id))
+  if (JSVAL_IS_STRING (id_val))
   {
-    JSString *str = JS_ValueToString (cx, id);
-    char     *prop = str ? JS_GetStringBytes (str) : "";
+    JSString *str = JS_ValueToString (cx, id_val);
+    char     *prop = str ? SE_JS_ENCODE_STRING (cx, str) : "";
     GList    *n;
 
     logprintf ("script_engine: looking for generic property '%s' in '%s'\n",
@@ -749,6 +765,9 @@ generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
       if (!strcasecmp (p->id, prop))
       {
 	*vp = get_prop_jsval (se, o, p);
+        if (str)
+          SE_JS_FREE_ENCODED_STRING (cx, prop);
+
 	return JS_TRUE;
       }
     }
@@ -760,18 +779,25 @@ generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
       se_o_t *p = (se_o_t *) n->data;
       if (!strcasecmp (p->id, prop))
       {
-	static jsval prop = 0;
-	if (!prop)
-	  prop = se_js_string_val (cx, ".");
-	if (!generic_JSGetProperty (cx, p->obj, prop, vp))
+	jsval prop_val = se_js_string_val (cx, ".");
+        if (!SE_JS_CALL_PROPERTY_GETTER_WITH_JSVAL (generic_JSGetProperty, cx, p->obj, prop_val, vp))
 	  *vp = OBJECT_TO_JSVAL (p->obj);
+        if (str)
+          SE_JS_FREE_ENCODED_STRING (cx, prop);
+
 	return JS_TRUE;
       }
     }
 
-    if (*prop && o->parent)
-      return generic_JSGetProperty (cx, o->parent->obj, id, vp);
+    if (*prop && o->parent) {
+      JSBool ret = generic_JSGetProperty (cx, o->parent->obj, id, vp);
+      if (str)
+        SE_JS_FREE_ENCODED_STRING (cx, prop);
+      return ret;
+    }
 
+    if (str)
+      SE_JS_FREE_ENCODED_STRING (cx, prop);
     return JS_TRUE;
   }
 
@@ -780,9 +806,9 @@ generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
   return JS_FALSE;
 }
 
-static JSBool
-generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+SE_JS_PROPERTY_SETTER_DECLARE(generic_JSSetProperty)
 {
+  SE_JS_PROPERTY_SETTER_INIT_VARIABLES (id_val);
   se_t   *se = (se_t *) JS_GetContextPrivate(cx);
   se_o_t *o;
 
@@ -790,10 +816,10 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
   o = JS_GetPrivate (cx, obj);
 
-  if (JSVAL_IS_STRING (id))
+  if (JSVAL_IS_STRING (id_val))
   {
-    JSString *str = JS_ValueToString (cx, id);
-    char     *prop = JS_GetStringBytes (str);
+    JSString *str = JS_ValueToString (cx, id_val);
+    char     *prop = SE_JS_ENCODE_STRING (cx, str);
     GList    *n;
 
     logprintf ("script_engine: looking for generic property '%s' in '%s'\n",
@@ -808,6 +834,7 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
       {
 	if (!p->constant)
 	  set_prop_jsval (se, o, p, vp);
+        SE_JS_FREE_ENCODED_STRING (cx, prop);
 	return JS_TRUE;
       }
     }
@@ -819,15 +846,23 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
       se_o_t *p = (se_o_t *) n->data;
       if (!strcasecmp (p->id, prop))
       {
-	if (!generic_JSSetProperty (cx, p->obj, STRING_TO_JSVAL("."), vp))
+	static jsval dot = 0;
+	if (!dot)
+	  dot = se_js_string_val (cx, ".");
+        if (!SE_JS_CALL_PROPERTY_SETTER_WITH_JSVAL (generic_JSSetProperty, cx, p->obj, dot, vp))
 	  *vp = OBJECT_TO_JSVAL (p->obj);
 	return JS_TRUE;
       }
     }
 
     if (*prop && o->parent)
-      return generic_JSSetProperty (cx, o->parent->obj, id, vp);
+    {
+      JSBool ret = SE_JS_CALL_PROPERTY_SETTER_WITH_JSVAL (generic_JSSetProperty, cx, o->parent->obj, id_val, vp);
+      SE_JS_FREE_ENCODED_STRING (cx, prop);
+      return ret;
+    }
 
+    SE_JS_FREE_ENCODED_STRING (cx, prop);
     return JS_TRUE;
   }
 
@@ -841,14 +876,35 @@ static void generic_JSDestructor (JSContext *cx, JSObject *obj)
   se_log_fncall ("generic destructor");
 }
 
+#ifdef JSFUN_CONSTRUCTOR
+static JSBool generic_Constructor (JSContext* cx, uintN argc, jsval* vp)
+{
+  JSObject* obj = JS_NewObjectForConstructor (cx, vp);
+  if (!obj) {
+    JS_ReportError (cx, "Failed to create 'this' object");
+    return JS_FALSE;
+  }
+
+  JS_SET_RVAL (cx, vp, OBJECT_TO_JSVAL (obj));
+  return JS_TRUE;
+}
+#endif
+
 static JSClass generic_JSClass =
   {
     "view", JSCLASS_HAS_PRIVATE,
     JS_PropertyStub, JS_PropertyStub,
     generic_JSGetProperty, generic_JSSetProperty,
     JS_EnumerateStub, JS_ResolveStub,
+#ifndef JSFUN_CONSTRUCTOR
     JS_ConvertStub, generic_JSDestructor
   };
+#else
+    JS_ConvertStub, generic_JSDestructor,
+    NULL, NULL, NULL, generic_Constructor,
+    NULL, NULL, NULL, NULL
+  };
+#endif
 
 se_o_t *se_create_object (se_t *se, se_o_t *parent /* may be NULL */,
 			  const gchar *name, void *user_data,
@@ -1542,7 +1598,7 @@ se_t *se_new (void)
 {
   se_t    *se;
   static JSClass global_class = {
-    "global", JSCLASS_HAS_PRIVATE,
+    "global", JSCLASS_HAS_PRIVATE | JSCLASS_GLOBAL_FLAGS,
     JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,
     JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub
   };
@@ -1564,7 +1620,13 @@ se_t *se_new (void)
   se = malloc (sizeof (se_t));
 
   se->cx       = JS_NewContext (rt, 0x1000);
+#ifdef HAVE_COMPARTMENTS
+  se->global   = JS_NewCompartmentAndGlobalObject (se->cx, &global_class, NULL);
+  /* Select the compartment our global object is in */
+  JS_EnterCrossCompartmentCall (se->cx, se->global);
+#else
   se->global   = JS_NewObject (se->cx, &global_class, NULL, NULL);
+#endif
 
   pthread_mutexattr_t attr;
   pthread_mutexattr_init (&attr);
@@ -1596,12 +1658,12 @@ se_t *se_new (void)
 
   {
     static const se_f_def_t defs[] = {
-      { "exit", controls_exit, 0, 0, SE_GROUP_ENGINE, NULL, NULL },
-      { "js_console_show", show_js_console, 0, 0, SE_GROUP_DIALOGUE, NULL, NULL },
-      { "help", show_help, 0, 0, SE_GROUP_HIDDEN, NULL, NULL },
-      { "callback", js_callback, 0, 0, SE_GROUP_ENGINE, N_("Javascript"), NULL },
-      { "xine_cfg_get", js_xine_cfg_get, 0, 0, SE_GROUP_PROPERTIES, NULL, NULL },
-      { "xine_cfg_set", js_xine_cfg_set, 0, 0, SE_GROUP_PROPERTIES, NULL, NULL },
+      { "exit", controls_exit, 0, JSFUN_FAST_NATIVE, SE_GROUP_ENGINE, NULL, NULL },
+      { "js_console_show", show_js_console, 0, JSFUN_FAST_NATIVE, SE_GROUP_DIALOGUE, NULL, NULL },
+      { "help", show_help, 0, JSFUN_FAST_NATIVE, SE_GROUP_HIDDEN, NULL, NULL },
+      { "callback", js_callback, 0, JSFUN_FAST_NATIVE, SE_GROUP_ENGINE, N_("Javascript"), NULL },
+      { "xine_cfg_get", js_xine_cfg_get, 0, JSFUN_FAST_NATIVE, SE_GROUP_PROPERTIES, NULL, NULL },
+      { "xine_cfg_set", js_xine_cfg_set, 0, JSFUN_FAST_NATIVE, SE_GROUP_PROPERTIES, NULL, NULL },
       { NULL }
     };
     se_defuns (se, se->g, defs);
