@@ -34,6 +34,97 @@
 #include <jsapi.h>
 #include <pthread.h>
 
+#ifndef JSFUN_FAST_NATIVE
+// All natives are JSFastNative
+#define JSFUN_FAST_NATIVE 0
+#endif
+
+#ifdef HAVE_JS_GET_STRING_BYTES
+# define SE_JS_ENCODE_STRING(cx, str) JS_GetStringBytes(str)
+# define SE_JS_FREE_ENCODED_STRING(cx, str)
+#else
+# define SE_JS_ENCODE_STRING(cx, str) JS_EncodeString(cx, str)
+# define SE_JS_FREE_ENCODED_STRING(cx, str) JS_free(cx, str)
+#endif
+
+#ifdef HAVE_JS_NEW_DOUBLE
+# define SE_JS_NEW_NUMBER_VAL(cx, d, rval) \
+    jsdouble *new_d = JS_NewDouble (cx, d); \
+    rval = DOUBLE_TO_JSVAL (new_d)
+#else
+# define SE_JS_NEW_NUMBER_VAL(cx, d, rval) \
+    JS_NewNumberValue (cx, d, &rval)
+#endif
+
+#ifdef JS_PROPERTY_OP_HAS_ID_AS_JSID
+# define SE_JS_PROPERTY_GETTER_DECLARE(name) \
+    static JSBool \
+    name(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
+# define SE_JS_PROPERTY_GETTER_INIT_VARIABLES(idval) \
+    jsval idval; \
+    JS_IdToValue (cx, id, &idval)
+# define SE_JS_CALL_PROPERTY_GETTER_WITH_JSVAL(getter, cx, obj, propval, vp) __extension__ ({ \
+    jsid propid; \
+    JS_ValueToId (cx, propval, &propid); \
+    JSBool ret = getter (cx, obj, propid, vp); \
+    ret; \
+})
+#else
+# define SE_JS_PROPERTY_GETTER_DECLARE(name) \
+    static JSBool \
+    name(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+# define SE_JS_PROPERTY_GETTER_INIT_VARIABLES(idval) \
+    jsval idval = id
+# define SE_JS_CALL_PROPERTY_GETTER_WITH_JSVAL(getter, cx, obj, propval, vp) __extension__ ({ \
+    JSBool ret = getter (cx, obj, propval, vp); \
+    ret; \
+})
+#endif
+
+#ifdef HAVE_JS_STRICT_PROPERTY_OP
+# define SE_JS_PROPERTY_SETTER_DECLARE(name) \
+    static JSBool \
+    name(JSContext *cx, JSObject *obj, jsid id, JSBool strict, jsval *vp)
+# define SE_JS_PROPERTY_SETTER_INIT_VARIABLES(idval) \
+    jsval idval; \
+    JS_IdToValue (cx, id, &idval)
+# define SE_JS_CALL_PROPERTY_SETTER_WITH_JSVAL(setter, cx, obj, propval, vp) __extension__ ({ \
+    jsid propid; \
+    JS_ValueToId (cx, propval, &propid); \
+    JSBool ret = setter (cx, obj, propid, JS_FALSE, vp); \
+    ret; \
+})
+#elif JS_PROPERTY_OP_HAS_ID_AS_JSID
+# define SE_JS_PROPERTY_SETTER_DECLARE(name) \
+    static JSBool \
+    name(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
+# define SE_JS_PROPERTY_SETTER_INIT_VARIABLES(idval) \
+    jsval idval; \
+    JS_IdToValue (cx, id, &idval)
+# define SE_JS_CALL_PROPERTY_SETTER_WITH_JSVAL(setter, cx, obj, propval, vp) __extension__ ({ \
+    jsid propid; \
+    JS_ValueToId (cx, propval, &propid); \
+    JSBool ret = setter (cx, obj, propid, vp); \
+    ret; \
+})
+#else
+# define SE_JS_PROPERTY_SETTER_DECLARE(name) \
+    static JSBool \
+    name(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+# define SE_JS_PROPERTY_SETTER_INIT_VARIABLES(idval) \
+    jsval idval = id
+# define SE_JS_CALL_PROPERTY_SETTER_WITH_JSVAL(setter, cx, obj, propval, vp) __extension__ ({ \
+    JSBool ret = setter (cx, obj, propval, vp); \
+    ret; \
+})
+#endif
+
+#ifdef JSVAL_TO_DOUBLE_RETURNS_POINTER
+# define SE_JSVAL_TO_DOUBLE(v) *JSVAL_TO_DOUBLE(v)
+#else
+# define SE_JSVAL_TO_DOUBLE(v) JSVAL_TO_DOUBLE(v)
+#endif
+
 typedef int (*se_prop_int_cb_t) (void *user_data, char *property, int *num);
 typedef void (*se_print_cb_t) (void *user_data, const char *str, ...)
   __attribute__ ((format (printf, 2, 3)));
